@@ -2360,7 +2360,7 @@ R600DrawInit(ScreenPtr pScreen)
 	info->accel_state->vsync = FALSE;
 
     if (!exaDriverInit(pScreen, info->accel_state->exa)) {
-	xfree(info->accel_state->exa);
+	free(info->accel_state->exa);
 	return FALSE;
     }
 
